@@ -18,14 +18,19 @@
 
 import csv
 import io
-import xmlrpclib
 import yaml
+import sys
 
 from linaro_django_xmlrpc.models import ExposedAPI
 
 from django.db.models.fields import FieldDoesNotExist
 
-from lava_results_app.dbutils import export_testcase, testcase_export_fields
+from lava_results_app.dbutils import (
+    export_testcase,
+    testcase_export_fields,
+    export_testsuite,
+    testsuite_export_fields
+)
 from lava_results_app.models import (
     Query,
     QueryCondition,
@@ -36,8 +41,16 @@ from lava_results_app.models import (
     TestData,
     InvalidContentTypeError,
 )
+from lava_results_app.utils import get_testcases_with_limit
 from lava_scheduler_app.models import TestJob
 
+if sys.version_info[0] == 2:
+    # Python 2.x
+    import xmlrpclib
+elif sys.version_info[0] == 3:
+    # For Python 3.0 and later
+    import xmlrpc.client as xmlrpclib
+
 
 class ResultsAPI(ExposedAPI):
 
@@ -383,11 +396,98 @@ class ResultsAPI(ExposedAPI):
 
         return output.getvalue()
 
-    def get_testsuite_results_yaml(self, job_id, suite_name):
+    def get_testjob_suites_list_csv(self, job_id):
         """
         Name
         ----
-        `get_testsuite_results_yaml` (`job_id`, `suite_name`)
+        `get_testjob_suites_list_csv` (`job_id`)
+
+        Description
+        -----------
+        Get the test suites list from job results of given job id in CSV format.
+
+        Arguments
+        ---------
+        `job_id`: string
+            Job id for which the test suites are required.
+
+        Return value
+        ------------
+        This function returns an XML-RPC structures of test suites list from
+        job results in CSV format, provided the user is authenticated with an
+        username and token.
+        """
+
+        self._authenticate()
+        if not job_id:
+            raise xmlrpclib.Fault(400, "Bad request: TestJob id was not "
+                                  "specified.")
+        try:
+            job = TestJob.get_by_job_number(job_id)
+            if not job.can_view(self.user):
+                raise xmlrpclib.Fault(
+                    401, "Permission denied for user to job %s" % job_id)
+            output = io.BytesIO()
+            writer = csv.DictWriter(
+                output,
+                quoting=csv.QUOTE_ALL,
+                extrasaction='ignore',
+                fieldnames=testsuite_export_fields())
+            writer.writeheader()
+            for test_suite in job.testsuite_set.all():
+                writer.writerow(export_testsuite(test_suite))
+
+        except TestJob.DoesNotExist:
+            raise xmlrpclib.Fault(404, "Specified job not found.")
+
+        return output.getvalue()
+
+    def get_testjob_suites_list_yaml(self, job_id):
+        """
+        Name
+        ----
+        `get_testjob_suites_list_yaml` (`job_id`)
+
+        Description
+        -----------
+        Get the test suites list from job results of given job id in YAML format.
+
+        Arguments
+        ---------
+        `job_id`: string
+            Job id for which the test suites are required.
+
+        Return value
+        ------------
+        This function returns an XML-RPC structures of test suites list from
+        job results in YAML format, provided the user is authenticated with an
+        username and token.
+        """
+
+        self._authenticate()
+        if not job_id:
+            raise xmlrpclib.Fault(400, "Bad request: TestJob id was not "
+                                  "specified.")
+        try:
+            job = TestJob.get_by_job_number(job_id)
+            if not job.can_view(self.user):
+                raise xmlrpclib.Fault(
+                    401, "Permission denied for user to job %s" % job_id)
+            yaml_list = []
+            for test_suite in job.testsuite_set.all():
+                yaml_list.append(export_testsuite(test_suite))
+
+        except TestJob.DoesNotExist:
+            raise xmlrpclib.Fault(404, "Specified job not found.")
+
+        return yaml.dump(yaml_list)
+
+    def get_testsuite_results_yaml(self, job_id, suite_name, limit=None,
+                                   offset=None):
+        """
+        Name
+        ----
+        `get_testsuite_results_yaml` (`job_id`, `suite_name`, `limit=None`, `offset=None`)
 
         Description
         -----------
@@ -399,6 +499,10 @@ class ResultsAPI(ExposedAPI):
             Job id for which the results are required.
         `suite_name`: string
             Name of the suite for which the results are required.
+        `limit`: int
+            Limit the number of test cases fetched.
+        `offset`: int
+            Start fetching test cases from a specific point.
 
         Return value
         ------------
@@ -417,7 +521,8 @@ class ResultsAPI(ExposedAPI):
                     401, "Permission denied for user to job %s" % job_id)
             yaml_list = []
             test_suite = job.testsuite_set.get(name=suite_name)
-            for test_case in test_suite.testcase_set.all():
+            for test_case in get_testcases_with_limit(test_suite,
+                                                      limit, offset):
                 yaml_list.append(export_testcase(test_case))
 
         except TestJob.DoesNotExist:
@@ -427,11 +532,12 @@ class ResultsAPI(ExposedAPI):
 
         return yaml.dump(yaml_list)
 
-    def get_testsuite_results_csv(self, job_id, suite_name):
+    def get_testsuite_results_csv(self, job_id, suite_name, limit=None,
+                                  offset=None):
         """
         Name
         ----
-        `get_testsuite_results_csv` (`job_id`, `suite_name`)
+        `get_testsuite_results_csv` (`job_id`, `suite_name`, `limit=None`, `offset=None`)
 
         Description
         -----------
@@ -443,6 +549,10 @@ class ResultsAPI(ExposedAPI):
             Job id for which the results are required.
         `suite_name`: string
             Name of the suite for which the results are required.
+        `limit`: int
+            Limit the number of test cases fetched.
+        `offset`: int
+            Start fetching test cases from a specific point.
 
         Return value
         ------------
@@ -467,7 +577,7 @@ class ResultsAPI(ExposedAPI):
                 fieldnames=testcase_export_fields())
             writer.writeheader()
             test_suite = job.testsuite_set.get(name=suite_name)
-            for row in test_suite.testcase_set.all():
+            for row in get_testcases_with_limit(test_suite, limit, offset):
                 writer.writerow(export_testcase(row))
 
         except TestJob.DoesNotExist:
@@ -477,6 +587,49 @@ class ResultsAPI(ExposedAPI):
 
         return output.getvalue()
 
+    def get_testsuite_results_count(self, job_id, suite_name):
+        """
+        Name
+        ----
+        `get_testsuite_results_count` (`job_id`, `suite_name`)
+
+        Description
+        -----------
+        Get the count of test cases in test suite.
+
+        Arguments
+        ---------
+        `job_id`: string
+            Job id for which the results are required.
+        `suite_name`: string
+            Name of the suite for which the test case count is required.
+
+        Return value
+        ------------
+        This function returns a count of test cases in particular test suite,
+        provided the user is authenticated with an username and token.
+        """
+
+        self._authenticate()
+        if not job_id:
+            raise xmlrpclib.Fault(400, "Bad request: TestJob id was not "
+                                  "specified.")
+        try:
+            job = TestJob.get_by_job_number(job_id)
+            if not job.can_view(self.user):
+                raise xmlrpclib.Fault(
+                    401, "Permission denied for user to job %s" % job_id)
+
+            test_suite = job.testsuite_set.get(name=suite_name)
+            test_case_count = test_suite.testcase_set.all().count()
+
+        except TestJob.DoesNotExist:
+            raise xmlrpclib.Fault(404, "Specified job not found.")
+        except TestSuite.DoesNotExist:
+            raise xmlrpclib.Fault(404, "Specified test suite not found.")
+
+        return test_case_count
+
     def get_testcase_results_yaml(self, job_id, suite_name, case_name):
         """
         Name
