@@ -34,7 +34,7 @@ class Migration(migrations.Migration):
             name='DefaultDeviceOwner',
             fields=[
                 ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
-                ('default_owner', models.BooleanField(default=False, unique=True, verbose_name=b'Default owner of unrestricted devices')),
+                ('default_owner', models.BooleanField(default=False, unique=True, verbose_name='Default owner of unrestricted devices')),
                 ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
             ],
             options={
@@ -48,8 +48,8 @@ class Migration(migrations.Migration):
                 ('hostname', models.CharField(max_length=200, serialize=False, verbose_name='Hostname', primary_key=True)),
                 ('device_version', models.CharField(default=None, max_length=200, null=True, verbose_name='Device Version', blank=True)),
                 ('description', models.TextField(default=None, max_length=200, null=True, verbose_name='Device Description', blank=True)),
-                ('status', models.IntegerField(default=1, verbose_name='Device status', choices=[(0, b'Offline'), (1, b'Idle'), (2, b'Running'), (3, b'Going offline'), (4, b'Retired'), (5, b'Reserved')])),
-                ('health_status', models.IntegerField(default=0, verbose_name='Device Health', choices=[(0, b'Unknown'), (1, b'Pass'), (2, b'Fail'), (3, b'Looping')])),
+                ('status', models.IntegerField(default=1, verbose_name='Device status', choices=[(0, 'Offline'), (1, 'Idle'), (2, 'Running'), (3, 'Going offline'), (4, 'Retired'), (5, 'Reserved')])),
+                ('health_status', models.IntegerField(default=0, verbose_name='Device Health', choices=[(0, 'Unknown'), (1, 'Pass'), (2, 'Fail'), (3, 'Looping')])),
             ],
             options={
                 'abstract': False,
@@ -61,8 +61,8 @@ class Migration(migrations.Migration):
             fields=[
                 ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                 ('created_on', models.DateTimeField(auto_now_add=True)),
-                ('old_state', models.IntegerField(choices=[(0, b'Offline'), (1, b'Idle'), (2, b'Running'), (3, b'Going offline'), (4, b'Retired'), (5, b'Reserved')])),
-                ('new_state', models.IntegerField(choices=[(0, b'Offline'), (1, b'Idle'), (2, b'Running'), (3, b'Going offline'), (4, b'Retired'), (5, b'Reserved')])),
+                ('old_state', models.IntegerField(choices=[(0, 'Offline'), (1, 'Idle'), (2, 'Running'), (3, 'Going offline'), (4, 'Retired'), (5, 'Reserved')])),
+                ('new_state', models.IntegerField(choices=[(0, 'Offline'), (1, 'Idle'), (2, 'Running'), (3, 'Going offline'), (4, 'Retired'), (5, 'Reserved')])),
                 ('message', models.TextField(null=True, blank=True)),
                 ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, blank=True, to=settings.AUTH_USER_MODEL, null=True)),
             ],
@@ -76,7 +76,7 @@ class Migration(migrations.Migration):
                 ('name', models.SlugField(serialize=False, primary_key=True)),
                 ('health_check_job', models.TextField(default=None, null=True, blank=True)),
                 ('display', models.BooleanField(default=True, help_text=b"Should this be displayed in the GUI or not. This can be useful if you are removing all devices of this type but don't want to loose the test results generated by the devices.")),
-                ('owners_only', models.BooleanField(default=False, help_text=b'Hide this device type for all users except owners of devices of this type.')),
+                ('owners_only', models.BooleanField(default=False, help_text='Hide this device type for all users except owners of devices of this type.')),
             ],
             options={
             },
@@ -127,24 +127,24 @@ class Migration(migrations.Migration):
                 ('submit_time', models.DateTimeField(auto_now_add=True, verbose_name='Submit time')),
                 ('start_time', models.DateTimeField(verbose_name='Start time', null=True, editable=False, blank=True)),
                 ('end_time', models.DateTimeField(verbose_name='End time', null=True, editable=False, blank=True)),
-                ('status', models.IntegerField(default=0, verbose_name='Status', choices=[(0, b'Submitted'), (1, b'Running'), (2, b'Complete'), (3, b'Incomplete'), (4, b'Canceled'), (5, b'Canceling')])),
-                ('priority', models.IntegerField(default=50, verbose_name='Priority', choices=[(0, b'Low'), (50, b'Medium'), (100, b'High')])),
+                ('status', models.IntegerField(default=0, verbose_name='Status', choices=[(0, 'Submitted'), (1, 'Running'), (2, 'Complete'), (3, 'Incomplete'), (4, 'Canceled'), (5, 'Canceling')])),
+                ('priority', models.IntegerField(default=50, verbose_name='Priority', choices=[(0, 'Low'), (50, 'Medium'), (100, 'High')])),
                 ('definition', models.TextField(editable=False)),
                 ('original_definition', models.TextField(editable=False, blank=True)),
                 ('multinode_definition', models.TextField(editable=False, blank=True)),
                 ('vmgroup_definition', models.TextField(editable=False, blank=True)),
                 ('admin_notifications', models.TextField(editable=False, blank=True)),
-                ('log_file', models.FileField(default=None, null=True, upload_to=b'lava-logs', blank=True)),
+                ('log_file', models.FileField(default=None, null=True, upload_to='lava-logs', blank=True)),
                 ('failure_comment', models.TextField(null=True, blank=True)),
-                ('_results_link', models.CharField(default=None, max_length=400, null=True, db_column=b'results_link', blank=True)),
-                ('_results_bundle', models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, db_column=b'results_bundle_id', blank=True, to='dashboard_app.Bundle')),
-                ('actual_device', models.ForeignKey(related_name=b'+', default=None, blank=True, to='lava_scheduler_app.Device', null=True)),
-                ('failure_tags', models.ManyToManyField(related_name=b'failure_tags', to='lava_scheduler_app.JobFailureTag', blank=True)),
+                ('_results_link', models.CharField(default=None, max_length=400, null=True, db_column='results_link', blank=True)),
+                ('_results_bundle', models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, db_column='results_bundle_id', blank=True, to='dashboard_app.Bundle')),
+                ('actual_device', models.ForeignKey(related_name='+', default=None, blank=True, to='lava_scheduler_app.Device', null=True)),
+                ('failure_tags', models.ManyToManyField(related_name='failure_tags', to='lava_scheduler_app.JobFailureTag', blank=True)),
                 ('group', models.ForeignKey(blank=True, to='auth.Group', null=True)),
-                ('requested_device', models.ForeignKey(related_name=b'+', default=None, blank=True, to='lava_scheduler_app.Device', null=True)),
-                ('requested_device_type', models.ForeignKey(related_name=b'+', default=None, blank=True, to='lava_scheduler_app.DeviceType', null=True)),
+                ('requested_device', models.ForeignKey(related_name='+', default=None, blank=True, to='lava_scheduler_app.Device', null=True)),
+                ('requested_device_type', models.ForeignKey(related_name='+', default=None, blank=True, to='lava_scheduler_app.DeviceType', null=True)),
                 ('submit_token', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, blank=True, to='linaro_django_xmlrpc.AuthToken', null=True)),
-                ('submitter', models.ForeignKey(related_name=b'+', verbose_name='Submitter', to=settings.AUTH_USER_MODEL)),
+                ('submitter', models.ForeignKey(related_name='+', verbose_name='Submitter', to=settings.AUTH_USER_MODEL)),
                 ('tags', models.ManyToManyField(to='lava_scheduler_app.Tag', blank=True)),
                 ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
             ],
@@ -157,7 +157,7 @@ class Migration(migrations.Migration):
             name='TestJobUser',
             fields=[
                 ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
-                ('is_favorite', models.BooleanField(default=False, verbose_name=b'Favorite job')),
+                ('is_favorite', models.BooleanField(default=False, verbose_name='Favorite job')),
                 ('test_job', models.ForeignKey(to='lava_scheduler_app.TestJob')),
                 ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
             ],
@@ -170,7 +170,7 @@ class Migration(migrations.Migration):
             fields=[
                 ('hostname', models.CharField(primary_key=True, default=None, serialize=False, editable=False, max_length=200, verbose_name='Hostname')),
                 ('rpc2_url', models.CharField(default=None, max_length=200, blank=True, help_text=b"Corresponds to the master node's RPC2 url. Does not have any impact when set on a worker node.", null=True, verbose_name='Master RPC2 URL')),
-                ('display', models.BooleanField(default=True, help_text=b'Should this be displayed in the GUI or not. This will be useful when a worker needs to be removed but still linked device status transitions and devices should be intact.')),
+                ('display', models.BooleanField(default=True, help_text='Should this be displayed in the GUI or not. This will be useful when a worker needs to be removed but still linked device status transitions and devices should be intact.')),
                 ('ip_address', models.CharField(default=None, editable=False, max_length=20, blank=True, null=True, verbose_name='IP Address')),
                 ('is_master', models.BooleanField(default=False, verbose_name='Is Master?')),
                 ('description', models.TextField(default=None, max_length=200, null=True, verbose_name='Worker Description', blank=True)),
@@ -193,7 +193,7 @@ class Migration(migrations.Migration):
         migrations.AddField(
             model_name='devicestatetransition',
             name='device',
-            field=models.ForeignKey(related_name=b'transitions', to='lava_scheduler_app.Device'),
+            field=models.ForeignKey(related_name='transitions', to='lava_scheduler_app.Device'),
             preserve_default=True,
         ),
         migrations.AddField(
@@ -205,7 +205,7 @@ class Migration(migrations.Migration):
         migrations.AddField(
             model_name='device',
             name='current_job',
-            field=models.ForeignKey(related_name=b'+', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to='lava_scheduler_app.TestJob', unique=True),
+            field=models.ForeignKey(related_name='+', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to='lava_scheduler_app.TestJob', unique=True),
             preserve_default=True,
         ),
         migrations.AddField(
@@ -223,19 +223,19 @@ class Migration(migrations.Migration):
         migrations.AddField(
             model_name='device',
             name='last_health_report_job',
-            field=models.ForeignKey(related_name=b'+', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to='lava_scheduler_app.TestJob', unique=True),
+            field=models.ForeignKey(related_name='+', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to='lava_scheduler_app.TestJob', unique=True),
             preserve_default=True,
         ),
         migrations.AddField(
             model_name='device',
             name='physical_group',
-            field=models.ForeignKey(related_name=b'physical-group', default=None, blank=True, to='auth.Group', null=True, verbose_name='Group with physical access'),
+            field=models.ForeignKey(related_name='physical-group', default=None, blank=True, to='auth.Group', null=True, verbose_name='Group with physical access'),
             preserve_default=True,
         ),
         migrations.AddField(
             model_name='device',
             name='physical_owner',
-            field=models.ForeignKey(related_name=b'physical-owner', default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='User with physical access'),
+            field=models.ForeignKey(related_name='physical-owner', default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='User with physical access'),
             preserve_default=True,
         ),
         migrations.AddField(
