@@ -1,12 +1,8 @@
 import os
 import sys
 import yaml
-import json
 import logging
-import cStringIO
-import xmlrpclib
 import unittest
-from django.test import TransactionTestCase
 from django.test.client import Client
 from django.contrib.auth.models import Permission, User
 from django.utils import timezone
@@ -19,14 +15,18 @@ from lava_scheduler_app.models import (
     Alias,
 )
 from lava_scheduler_app.schema import validate_submission, validate_device, SubmissionException
-from lava_scheduler_app.dbutils import (
-    testjob_submission, get_job_queue,
-    find_device_for_job,
-    get_available_devices,
-)
 from lava_scheduler_app.tests.test_submission import ModelFactory, TestCaseWithFactory
 # pylint: disable=invalid-name
 
+if sys.version_info[0] == 2:
+    # Python 2.x
+    from cStringIO import StringIO
+    import xmlrpclib
+elif sys.version_info[0] == 3:
+    # For Python 3.0 and later
+    from io import BytesIO as StringIO
+    import xmlrpc.client as xmlrpclib
+
 
 # Based on http://www.technobabble.dk/2008/apr/02/xml-rpc-dispatching-through-django-test-client/
 class TestTransport(xmlrpclib.Transport, object):
@@ -46,7 +46,7 @@ class TestTransport(xmlrpclib.Transport, object):
         self.verbose = verbose
         response = self.client.post(
             handler, request_body, content_type="text/xml")
-        res = cStringIO.StringIO(response.content)
+        res = StringIO(response.content)
         res.seek(0)
         return self.parse_response(res)
 
@@ -94,16 +94,9 @@ class TestSchedulerAPI(TestCaseWithFactory):  # pylint: disable=too-many-ancesto
         device.save()
         server = self.server_proxy('test', 'test')
         self.assertEqual(
-            {'status': 'idle', 'job': None, 'offline_since': None, 'hostname': 'black01',
+            {'status': 'Idle', 'job': None, 'offline_since': None, 'hostname': 'black01',
                 'offline_by': None, 'is_pipeline': False},
             server.scheduler.get_device_status('black01'))
-        offline_device = self.factory.make_device(device_type=device_type, hostname="black02", status=Device.OFFLINE)
-        offline_device.save()
-        self.assertEqual(
-            {'status': 'offline', 'job': None, 'offline_since': '', 'hostname': 'black02',
-                'offline_by': '', 'is_pipeline': False},
-            server.scheduler.get_device_status('black02')
-        )
 
     def test_type_aliases(self):
         aliases = DeviceType.objects.filter(aliases__name__contains='black')
@@ -126,64 +119,6 @@ class TestSchedulerAPI(TestCaseWithFactory):  # pylint: disable=too-many-ancesto
         }
         self.assertEqual(retval, {'black': []})
 
-    # comment out the decorator to run this queue timing test
-    @unittest.skip('Developer only - timing test')
-    def test_queueing(self):
-        """
-        uses stderr to avoid buffered prints
-        Expect the test itself to take <30s and
-        the gap between jobs submitted and end being ~500ms
-        Most of the time is spent setting up the database
-        and submitting all the test jobs.
-        """
-        sys.stderr.write(timezone.now(), "start")
-        user = self.factory.ensure_user('test', 'e@mail.invalid', 'test')
-        user.user_permissions.add(
-            Permission.objects.get(codename='add_testjob'))
-        user.save()
-        device_type = self.factory.make_device_type('beaglebone-black')
-        device = self.factory.make_device(device_type=device_type, hostname="black01")
-        device.save()
-        device_type = self.factory.make_device_type('wandboard')
-        count = 1
-        while count < 100:
-            suffix = "{:02d}".format(count)
-            device = self.factory.make_device(device_type=device_type, hostname="imx6q-%s" % suffix)
-            device.save()
-            count += 1
-        sys.stderr.write(timezone.now(), "%d dummy devices created" % count)
-        device_list = list(get_available_devices())
-        sys.stderr.write(timezone.now(), "%d available devices" % len(device_list))
-        filename = os.path.join(os.path.dirname(__file__), 'sample_jobs', 'master-check.json')
-        self.assertTrue(os.path.exists(filename))
-        with open(filename, 'r') as json_file:
-            definition = json_file.read()
-        count = 0
-        # each 1000 more can take ~15s in the test.
-        while count < 1000:
-            # simulate API submission
-            job = testjob_submission(definition, user)
-            self.assertFalse(job.health_check)
-            count += 1
-        sys.stderr.write(timezone.now(), "%d jobs submitted" % count)
-        jobs = list(get_job_queue())
-        self.assertIsNotNone(jobs)
-        sys.stderr.write(timezone.now(), "Finding devices for jobs.")
-        for job in jobs:
-            # this needs to stay as a tight loop to cope with load
-            device = find_device_for_job(job, device_list)
-            if device:
-                sys.stderr.write(timezone.now(), "[%d] allocated %s" % (job.id, device))
-                device_list.remove(device)
-        sys.stderr.write(timezone.now(), "end")
-
-
-class TransactionTestCaseWithFactory(TransactionTestCase):
-
-    def setUp(self):
-        TransactionTestCase.setUp(self)
-        self.factory = ModelFactory()
-
 
 class TestVoluptuous(unittest.TestCase):
 
@@ -433,3 +368,143 @@ secrets:
   username: secret
 """
         self.assertRaises(SubmissionException, validate_submission, yaml.load(secrets))
+
+    def test_multinode(self):
+        # Without protocols
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+"""
+        self.assertTrue(validate_submission(yaml.load(data)))
+
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols: {}
+"""
+        self.assertTrue(validate_submission(yaml.load(data)))
+
+        # With a valid multinode protocol
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest: {}
+      host: {}
+"""
+        self.assertTrue(validate_submission(yaml.load(data)))
+
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest:
+        host_role: host
+        expect_role: host
+      host: {}
+"""
+        self.assertTrue(validate_submission(yaml.load(data)))
+
+        # invalid host_role or expect_role
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest:
+        host_role: server
+      host: {}
+"""
+        self.assertRaises(SubmissionException, validate_submission, yaml.load(data))
+
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest:
+        host_role: host
+        expect_role: server
+      host: {}
+"""
+        self.assertRaises(SubmissionException, validate_submission, yaml.load(data))
+
+        # host_role without expect_role
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest:
+        host_role: host
+      host: {}
+"""
+        self.assertRaises(SubmissionException, validate_submission, yaml.load(data))
+
+        # expect_role without host_role
+        data = """
+job_name: test
+visibility: public
+timeouts:
+  job:
+    minutes: 10
+  action:
+    minutes: 5
+actions: []
+protocols:
+  lava-multinode:
+    roles:
+      guest:
+        expect_role: host
+      host: {}
+"""
+        self.assertRaises(SubmissionException, validate_submission, yaml.load(data))
