@@ -19,12 +19,6 @@ from lava_scheduler_app.utils import (
     split_multinode_yaml,
 )
 from lava_scheduler_app.tests.test_submission import ModelFactory, TestCaseWithFactory
-from lava_scheduler_app.dbutils import (
-    testjob_submission,
-    find_device_for_job,
-    initiate_health_check_job,
-    end_job,
-)
 from lava_scheduler_app.schema import (
     validate_submission,
     validate_device,
@@ -96,9 +90,6 @@ class YamlFactory(ModelFactory):
         data.update(kw)
         return data
 
-    def make_job_json(self, **kw):
-        return self.make_job_yaml(**kw)
-
     def make_job_yaml(self, **kw):
         return yaml.safe_dump(self.make_job_data(**kw))
 
@@ -139,17 +130,18 @@ class PipelineDeviceTags(TestCaseWithFactory):
     def test_no_tags(self):
         self.factory.make_device(self.device_type, 'fakeqemu3')
         TestJob.from_yaml_and_user(
-            self.factory.make_job_json(),
+            self.factory.make_job_yaml(),
             self.factory.make_user())
 
     def test_priority(self):
         self.factory.make_device(self.device_type, 'fakeqemu3')
         job = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(),
+            self.factory.make_job_yaml(),
             self.factory.make_user())
         self.assertEqual(TestJob.LOW, job.priority)
 
     def test_yaml_device_tags(self):
+        Device.CONFIG_PATH = os.path.join(os.getcwd(), 'lava_scheduler_app', 'tests', 'devices')
         Tag.objects.all().delete()
         tag_list = [
             self.factory.ensure_tag('usb'),
@@ -173,7 +165,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
         self.assertRaises(
             yaml.YAMLError,
             TestJob.from_yaml_and_user,
-            self.factory.make_job_json(tags=['tag1', 'tag2']),
+            self.factory.make_job_yaml(tags=['tag1', 'tag2']),
             self.factory.make_user())
 
     def test_from_yaml_unsupported_tags(self):
@@ -182,7 +174,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
         self.factory.ensure_tag('sata')
         try:
             TestJob.from_yaml_and_user(
-                self.factory.make_job_json(tags=['usb', 'sata']),
+                self.factory.make_job_yaml(tags=['usb', 'sata']),
                 self.factory.make_user())
         except DevicesUnavailableException:
             pass
@@ -196,7 +188,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
         ]
         self.factory.make_device(self.device_type, hostname='fakeqemu1', tags=tag_list)
         job = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(tags=['tag1', 'tag2']),
+            self.factory.make_job_yaml(tags=['tag1', 'tag2']),
             self.factory.make_user())
         self.assertEqual(
             set(tag.name for tag in job.tags.all()), {'tag1', 'tag2'})
@@ -206,10 +198,10 @@ class PipelineDeviceTags(TestCaseWithFactory):
         tags = list(Tag.objects.filter(name='tag'))
         self.factory.make_device(device_type=self.device_type, hostname="fakeqemu1", tags=tags)
         job1 = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(tags=['tag']),
+            self.factory.make_job_yaml(tags=['tag']),
             self.factory.make_user())
         job2 = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(tags=['tag']),
+            self.factory.make_job_yaml(tags=['tag']),
             self.factory.make_user())
         self.assertEqual(
             set(tag.pk for tag in job1.tags.all()),
@@ -230,7 +222,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
         tag_list.append(self.factory.ensure_tag('unique_tag'))
         self.factory.make_device(device_type=self.device_type, hostname="fakeqemu2", tags=tag_list)
         job = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(tags=['common_tag1', 'common_tag2', 'unique_tag']),
+            self.factory.make_job_yaml(tags=['common_tag1', 'common_tag2', 'unique_tag']),
             self.factory.make_user())
         self.assertEqual(
             set(tag for tag in job.tags.all()),
@@ -248,99 +240,17 @@ class TestPipelineSubmit(TestCaseWithFactory):
         self.factory.make_device(device_type=self.device_type, hostname="fakeqemu3")
 
     def test_from_yaml_and_user_sets_definition(self):
-        definition = self.factory.make_job_json()
+        definition = self.factory.make_job_yaml()
         job = TestJob.from_yaml_and_user(definition, self.factory.make_user())
         self.assertEqual(definition, job.definition)
 
     def test_from_yaml_and_user_sets_submitter(self):
         user = self.factory.make_user()
         job = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(), user)
+            self.factory.make_job_yaml(), user)
         self.assertEqual(user, job.submitter)
         self.assertFalse(job.health_check)
 
-    def test_pipeline_health_check(self):
-        user = self.factory.make_user()
-        device = Device.objects.get(hostname='fakeqemu1')
-        job = testjob_submission(self.factory.make_job_yaml(), user, check_device=None)
-        self.assertEqual(user, job.submitter)
-        self.assertFalse(job.health_check)
-        self.assertIsNone(job.requested_device)
-        job = testjob_submission(self.factory.make_job_yaml(), user, check_device=device)
-        self.assertEqual(user, job.submitter)
-        self.assertTrue(job.health_check)
-        self.assertEqual(job.requested_device, device)
-        self.assertIsNone(job.actual_device)
-
-    def test_pipeline_health_assignment(self):
-        user = self.factory.make_user()
-        device1 = Device.objects.get(hostname='fakeqemu1')
-        self.assertTrue(device1.is_pipeline)
-        device2 = self.factory.make_device(device_type=device1.device_type, hostname='fakeqemu2')
-        self.assertTrue(device2.is_pipeline)
-        device3 = self.factory.make_device(device_type=device1.device_type, hostname='fakeqemu3')
-        self.assertTrue(device3.is_pipeline)
-        job1 = testjob_submission(self.factory.make_job_yaml(), user, check_device=device1)
-        job2 = testjob_submission(self.factory.make_job_yaml(), user, check_device=device2)
-        self.assertEqual(user, job1.submitter)
-        self.assertTrue(job1.health_check)
-        self.assertEqual(job1.requested_device, device1)
-        self.assertEqual(job2.requested_device, device2)
-        self.assertIsNone(job1.actual_device)
-        self.assertIsNone(job2.actual_device)
-        device_list = Device.objects.filter(device_type=device1.device_type)
-        count = 0
-        while True:
-            device_list.reverse()
-            assigned = find_device_for_job(job2, device_list)
-            if assigned != device2:
-                self.fail("[%d] invalid device assigment in health check." % count)
-            count += 1
-            if count > 100:
-                break
-        self.assertGreater(count, 100)
-
-    def test_multinode_tags_assignment(self):
-        # kvm-multinode.yaml contains notification settings for admin
-        user = self.factory.ensure_user('admin', 'admin@test.com', 'admin')
-        device1 = Device.objects.get(hostname='fakeqemu1')
-        client_tag_list = [
-            self.factory.ensure_tag('usb-flash'),
-            self.factory.ensure_tag('usb-eth')
-        ]
-        self.factory.make_device(device_type=self.device_type, hostname="fakeqemu1", tags=client_tag_list)
-        self.assertTrue(device1.is_pipeline)
-        server_tag_list = [
-            self.factory.ensure_tag('testtag')
-        ]
-        device2 = self.factory.make_device(device_type=device1.device_type, hostname='fakeqemu2', tags=server_tag_list)
-        self.assertTrue(device2.is_pipeline)
-        submission = yaml.load(open(
-            os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
-        job_list = testjob_submission(yaml.dump(submission), user)
-        self.assertEqual(len(job_list), 2)
-        client_job = None
-        server_job = None
-        for job in job_list:
-            job_def = yaml.load(job.definition)
-            if job_def['protocols'][MultinodeProtocol.name]['role'] == 'client':
-                client_job = job
-            elif job_def['protocols'][MultinodeProtocol.name]['role'] == 'server':
-                server_job = job
-            else:
-                self.fail("Unrecognised role in job")
-        self.assertEqual(set(client_job.tags.all()), set(client_tag_list))
-        self.assertEqual(set(server_job.tags.all()), set(server_tag_list))
-        self.assertIsNone(client_job.actual_device)
-        self.assertIsNone(server_job.actual_device)
-        device_list = Device.objects.filter(device_type=device1.device_type)
-        assigned = find_device_for_job(client_job, device_list)
-        self.assertEqual(assigned, device1)
-        self.assertEqual(set(device1.tags.all()), set(client_job.tags.all()))
-        assigned = find_device_for_job(server_job, device_list)
-        self.assertEqual(assigned, device2)
-        self.assertEqual(set(device2.tags.all()), set(server_job.tags.all()))
-
     def test_exclusivity(self):
         device = Device.objects.get(hostname="fakeqemu1")
         self.assertTrue(device.is_pipeline)
@@ -360,7 +270,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         device = Device.objects.get(hostname="fakeqemu1")
         user = self.factory.make_user()
         job = TestJob.from_yaml_and_user(
-            self.factory.make_job_json(), user)
+            self.factory.make_job_yaml(), user)
         job_def = yaml.load(job.definition)
         job_ctx = job_def.get('context', {})
         device_config = device.load_configuration(job_ctx)  # raw dict
@@ -472,7 +382,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
 
         boot_params = None
         for name, params in ((x, d[x])
-                             for x, d in ((d.keys()[0], d)
+                             for x, d in ((list(d.keys())[0], d)
                                           for d in data['actions'])):
             if name == 'boot':
                 boot_params = params
@@ -501,7 +411,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         user3 = self.factory.make_user()
 
         # public set in the YAML
-        yaml_str = self.factory.make_job_json()
+        yaml_str = self.factory.make_job_yaml()
         yaml_data = yaml.load(yaml_str)
         job = TestJob.from_yaml_and_user(
             yaml_str, user)
@@ -553,7 +463,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
     def test_compatibility(self):  # pylint: disable=too-many-locals
         user = self.factory.make_user()
         # public set in the YAML
-        yaml_str = self.factory.make_job_json()
+        yaml_str = self.factory.make_job_yaml()
         yaml_data = yaml.load(yaml_str)
         job = TestJob.from_yaml_and_user(
             yaml_str, user)
@@ -600,130 +510,13 @@ class TestPipelineSubmit(TestCaseWithFactory):
         self.assertIn('compatibility', description)
         self.assertGreaterEqual(description['compatibility'], BootQEMU.compatibility)
 
-    def test_device_maintenance_with_jobs(self):
-        user = self.factory.make_user()
-        job = TestJob.from_yaml_and_user(
-            self.factory.make_job_yaml(), user)
-        self.assertEqual(user, job.submitter)
-        self.assertFalse(job.health_check)
-        device1 = Device.objects.get(hostname='fakeqemu1')
-        device1.current_job = None
-        device1.status = Device.IDLE
-        device1.save()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.IDLE)
-        device1.put_into_maintenance_mode(user, 'unit test')
-        self.assertEqual(device1.status, Device.OFFLINE)
-        device1.put_into_online_mode(user, 'unit-test')
-        self.assertEqual(device1.status, Device.IDLE)
-        device1.current_job = job
-        device1.status = Device.RUNNING
-        device1.save()
-        device1.refresh_from_db()
-        device1.put_into_maintenance_mode(user, 'unit test')
-        self.assertIsNotNone(device1.current_job)
-        self.assertEqual(device1.status, Device.OFFLINING)
-        device1.put_into_online_mode(user, 'unit-test')
-        self.assertEqual(device1.status, Device.RUNNING)
-        device1.current_job = None
-        device1.status = Device.RETIRED
-        device1.save()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        device1.put_into_maintenance_mode(user, 'unit test')
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(device1.status, Device.RETIRED)
-        device1.put_into_online_mode(user, 'unit-test')
-        self.assertEqual(device1.status, Device.RETIRED)
-
-    def reset_job_device(self, job, device, job_status=TestJob.RUNNING, device_status=Device.RUNNING):
+    def reset_job_device(self, job, device, job_state=TestJob.STATE_RUNNING, device_state=Device.STATE_RUNNING):
         device.current_job = job
-        job.status = job_status
-        device.status = device_status
+        job.state = job_state
+        device.state = device_state
         job.save()
         device.save()
 
-    def test_job_ending(self):
-        user = self.factory.make_user()
-        job = TestJob.from_yaml_and_user(
-            self.factory.make_job_yaml(), user)
-        self.assertEqual(user, job.submitter)
-        self.assertFalse(job.health_check)
-        device1 = Device.objects.get(hostname='fakeqemu1')
-        device1.current_job = None
-        device1.status = Device.IDLE
-        device1.save()
-        self.assertEqual(device1.status, Device.IDLE)
-        self.assertEqual(job.status, TestJob.SUBMITTED)
-        job.actual_device = device1
-
-        self.reset_job_device(job, device1)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.RUNNING)
-        self.assertEqual(job.status, TestJob.RUNNING)
-
-        # standard complete test job
-        end_job(job, 'unit test')
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(job.status, TestJob.COMPLETE)
-
-        self.reset_job_device(job, device1)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.RUNNING)
-        self.assertEqual(job.status, TestJob.RUNNING)
-
-        # standard failed test job
-        end_job(job, 'unit test', TestJob.INCOMPLETE)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(job.status, TestJob.INCOMPLETE)
-
-        self.reset_job_device(job, device1, job_status=TestJob.CANCELING)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.RUNNING)
-        self.assertEqual(job.status, TestJob.CANCELING)
-
-        # cancelled test job
-        end_job(job, 'unit test', TestJob.CANCELING)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(job.status, TestJob.CANCELED)
-
-        self.reset_job_device(job, device1, device_status=Device.OFFLINING)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.OFFLINING)
-        self.assertEqual(job.status, TestJob.RUNNING)
-
-        # job ends when device is offlining
-        end_job(job, 'unit test')
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(device1.status, Device.OFFLINE)
-        self.assertEqual(job.status, TestJob.COMPLETE)
-
-        self.reset_job_device(job, device1, job_status=TestJob.CANCELING, device_status=Device.OFFLINING)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertEqual(device1.status, Device.OFFLINING)
-        self.assertEqual(job.status, TestJob.CANCELING)
-
-        # job cancelled when device is offlining
-        end_job(job, 'unit test', job_status=TestJob.CANCELING)
-        job.refresh_from_db()
-        device1.refresh_from_db()
-        self.assertIsNone(device1.current_job)
-        self.assertEqual(device1.status, Device.OFFLINE)
-        self.assertEqual(job.status, TestJob.CANCELED)
-
     def test_include_yaml_non_dict(self):
         include_data = ['value1', 'value2']
         self.assertRaises(
@@ -789,23 +582,7 @@ class TestExtendsSubmit(TestCaseWithFactory):
             os.path.exists(
                 os.path.join(
                     Device.HEALTH_CHECK_PATH, "%s.yaml" % device1.get_extends())))
-        self.assertIsNotNone(initiate_health_check_job(device1))
-
-    def test_health_checks_dbname(self):
-        device2 = Device.objects.get(hostname='juno-r2-01')
-        self.assertIsNotNone(device2.load_configuration(output_format='raw'))
-        self.assertEqual('juno', device2.get_extends())
-        self.assertIsNotNone(device2.HEALTH_CHECK_PATH)
-        self.assertTrue(
-            os.path.exists(
-                os.path.join(
-                    Device.HEALTH_CHECK_PATH, "%s.yaml" % device2.get_extends())))
-        # device_type specified in the job does not exist in the database
-        self.assertIsNone(initiate_health_check_job(device2))
-
-        device_type = self.factory.make_device_type(name='juno')
-        self.factory.make_device(device_type=device_type, hostname="juno-01")
-        self.assertIsNotNone(initiate_health_check_job(device2))
+        self.assertIsNotNone(device1.get_health_check())
 
 
 class TestYamlMultinode(TestCaseWithFactory):
@@ -843,8 +620,8 @@ class TestYamlMultinode(TestCaseWithFactory):
         self.assertEqual(len(jobs), 2)
         for role, job_list in jobs.items():
             for job in job_list:
+                del(job['protocols']['lava-multinode']['sub_id'])
                 yaml.dump(job)  # ensure the jobs can be serialised as YAML
-                role = job['protocols']['lava-multinode']['role']
                 if role == 'client':
                     self.assertEqual(job, yaml.load(open(client_check, 'r')))
                 if role == 'server':
@@ -947,7 +724,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                 'security_mirror': 'http://mirror.csclub.uwaterloo.ca/debian-security/', 'release': 'sid',
                 'distribution': 'debian', 'mirror': 'http://ftp.us.debian.org/debian/'}
         }
-        for role, _ in jobs.iteritems():
+        for role, _ in jobs.items():
             if role == 'server':
                 self.assertNotIn('lava-lxc', jobs[role][0]['protocols'])
             elif role == 'client':
@@ -973,7 +750,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                 'distribution': 'debian', 'mirror': 'http://mirror.bytemark.co.uk/debian',
                 'name': 'lxc-hikey-oe', 'release': 'jessie', 'template': 'debian'}
         }
-        for role, _ in jobs.iteritems():
+        for role, _ in jobs.items():
             if role == 'server':
                 self.assertIn('lava-lxc', jobs[role][0]['protocols'])
                 self.assertEqual(jobs[role][0]['protocols']['lava-lxc'], server_protocol_data['lava-lxc'])
@@ -1242,8 +1019,8 @@ class TestYamlMultinode(TestCaseWithFactory):
         self.assertNotIn(fakeqemu1, allowed_devices)
 
         # set one candidate device to RETIRED to force the bug
-        fakeqemu4.status = Device.RETIRED
-        fakeqemu4.save(update_fields=['status'])
+        fakeqemu4.health = Device.HEALTH_RETIRED
+        fakeqemu4.save(update_fields=['health'])
         # refresh the device_list
         device_list = Device.objects.filter(device_type=device_type, is_pipeline=True).order_by('hostname')
         allowed_devices = []
@@ -1257,7 +1034,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         except DevicesUnavailableException:
             self.assertEqual(len(allowed_devices), 2)
             self.assertIn(fakeqemu4, device_list)
-            self.assertIn(fakeqemu4.status, [Device.RETIRED])
+            self.assertEqual(fakeqemu4.health, Device.HEALTH_RETIRED)
         else:
             self.fail("Missed DevicesUnavailableException")
         allowed_devices = []
@@ -1271,7 +1048,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 
         # test improvement as there is no point wasting memory with a Query containing Retired.
         device_list = Device.objects.filter(
-            Q(device_type=device_type), Q(is_pipeline=True), ~Q(status=Device.RETIRED))
+            Q(device_type=device_type), Q(is_pipeline=True), ~Q(health=Device.HEALTH_RETIRED))
         allowed_devices.extend(_check_submit_to_device(list(device_list), user))
         self.assertEqual(len(allowed_devices), 2)
         self.assertIn(fakeqemu3, allowed_devices)
@@ -1435,11 +1212,11 @@ class VlanInterfaces(TestCaseWithFactory):
         submission = yaml.load(open(self.filename, 'r'))
         self.assertIn('protocols', submission)
         self.assertIn('lava-vland', submission['protocols'])
-        roles = [role for role, _ in submission['protocols']['lava-vland'].iteritems()]
+        roles = [role for role, _ in submission['protocols']['lava-vland'].items()]
         params = submission['protocols']['lava-vland']
         vlans = {}
         for role in roles:
-            for name, tags in params[role].iteritems():
+            for name, tags in params[role].items():
                 vlans[name] = tags
         self.assertIn('vlan_one', vlans)
         self.assertIn('vlan_two', vlans)
