@@ -1,25 +1,25 @@
 from django.conf.urls import url
 from lava_scheduler_app.views import (
     active_device_list, active_jobs, device_derestrict_device, device_detail,
-    device_edit_description, device_force_health_check,
-    device_health_history_log, device_list, device_looping_mode,
-    device_maintenance_mode, device_online, device_reports,
+    device_edit_description, device_health,
+    device_type_health_history_log,
+    mydevices_health_history_log, device_list, device_reports,
     device_restrict_device, device_type_detail, device_dictionary,
-    device_dictionary_plain, device_type_health_history_log,
-    device_type_reports, edit_transition, edit_worker_desc,
+    device_dictionary_plain,
+    device_type_reports, edit_worker_desc,
     failure_report, favorite_jobs,
     get_remote_definition, health_job_list, healthcheck, index,
     job_annotate_failure, job_cancel, job_change_priority, job_complete_log,
     job_definition, job_definition_plain, job_description_yaml, job_detail,
-    job_json, job_list,
+    job_list,
     job_log_file_plain, job_log_pipeline_incremental,
     job_pipeline_timing, job_resubmit, job_section_log, job_status,
     job_submit, job_toggle_favorite, lab_health,
     longest_jobs, multinode_job_definition, multinode_job_definition_plain,
-    mydevice_list, mydevices_health_history_log, myjobs, online_device_list,
+    mydevice_list, myjobs, online_device_list,
     passing_health_checks, queue, reports,
-    running, transition_detail, username_list_json,
-    worker_detail, worker_active, worker_maintenance, worker_retired, workers,
+    running, username_list_json,
+    worker_detail, worker_health, workers,
     download_device_type_template, similar_jobs,)
 
 
@@ -44,23 +44,8 @@ urlpatterns = [
     url(r'^allworkers$', workers, name='lava.scheduler.workers'),
     url(r'^worker/(?P<pk>[-_a-zA-Z0-9.]+)$', worker_detail,
         name='lava.scheduler.worker.detail'),
-    url(r'^worker/(?P<pk>[-_a-zA-Z0-9.]+)/active$', worker_active,
-        name='lava.scheduler.worker.active'),
-    url(r'^worker/(?P<pk>[-_a-zA-Z0-9.]+)/maintenance$', worker_maintenance,
-        name='lava.scheduler.worker.maintenance'),
-    url(r'^worker/(?P<pk>[-_a-zA-Z0-9.]+)/retired$', worker_retired,
-        name='lava.scheduler.worker.retired'),
-    url(r'^edit-transition', edit_transition,
-        name='lava.scheduler.edit_transition'),
-    url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/maintenance$', device_maintenance_mode,
-        name='lava.scheduler.device.maintenance'),
-    url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/forcehealthcheck$',
-        device_force_health_check,
-        name='lava.scheduler.device.forcehealthcheck'),
-    url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/looping$', device_looping_mode,
-        name='lava.scheduler.device.looping'),
-    url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/online$', device_online,
-        name='lava.scheduler.device.online'),
+    url(r'^worker/(?P<pk>[-_a-zA-Z0-9.]+)/health$', worker_health,
+        name='lava.scheduler.worker.health'),
     url(r'^labhealth/$', lab_health, name='lava.scheduler.labhealth'),
     url(r'^labhealth/device/(?P<pk>[-_a-zA-Z0-9.]+)$', health_job_list,
         name='lava.scheduler.labhealth.detail'),
@@ -101,7 +86,6 @@ urlpatterns = [
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/toggle_favorite$',
         job_toggle_favorite,
         name='lava.scheduler.job.toggle_favorite'),
-    url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/json$', job_json, name='lava.scheduler.job.json'),
     url(r'^job/(?P<pk>[0-9]+|[0-9]+\.[0-9]+)/log_pipeline_incremental$',
         job_log_pipeline_incremental,
         name='lava.scheduler.job.log_pipeline_incremental'),
@@ -120,8 +104,10 @@ urlpatterns = [
     url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/description$',
         device_edit_description,
         name='lava.scheduler.device.edit_description'),
-    url(r'^transition/(?P<pk>[0-9]+)$', transition_detail,
-        name='lava.scheduler.transition_detail'),
+
+    url(r'^device/(?P<pk>[-_a-zA-Z0-9.]+)/health$', device_health,
+        name='lava.scheduler.device.health'),
+
     url(r'^alldevices/active$', active_device_list,
         name='lava.scheduler.active_devices'),
     url(r'^alldevices/online$', online_device_list,
@@ -138,9 +124,6 @@ urlpatterns = [
     url(r'^queue$', queue, name='lava.scheduler.queue'),
     url(r'^healthcheck$', healthcheck, name='lava.scheduler.healthcheck'),
     url(r'^running$', running, name='lava.scheduler.running'),
-    url(r'^devicehealthhistory/device/(?P<pk>[-_a-zA-Z0-9.]+)',
-        device_health_history_log,
-        name='lava.scheduler.device_health_history_log'),
     url(r'^dthealthhistory/device_type/(?P<pk>[-_a-zA-Z0-9.]+)',
         device_type_health_history_log,
         name='lava.scheduler.device_type_health_history_log'),
