@@ -66,8 +66,8 @@ def handler(request, mapper, help_view):  # pylint: disable=too-many-return-stat
                 return HttpResponse(
                     "Unsupported HTTP_AUTHORIZATION header, only Basic scheme is supported", status=400)
             try:
-                decoded_value = base64.standard_b64decode(value)
-            except TypeError:
+                decoded_value = base64.standard_b64decode(value).decode("utf-8")
+            except (TypeError, UnicodeDecodeError):
                 return HttpResponse("Corrupted HTTP_AUTHORIZATION header, bad base64 encoding", status=400)
             try:
                 username, secret = decoded_value.split(":", 1)
